**free

// RPG BATTLE DEV 2024 - Manche 1 - Epreuve 2 - Du 6 au 8 Novembre 2024

// Contexte :
//  Faire un programme de mise en forme des données de log
//  Le fichier base de données TEAMLIB/LOG contient une liste des logs

// Objectif :
//  Afficher les logs présents dans le fichier LOG et les afficher (DSPLY) de manière descendante
// (du plus récent au plus ancient)
//  Exemple de résultat (format à respecter):
//   DSPLY 11 Novembre 2024 - 17h12mn00s - Avertissement
//   DSPLY 07 Octobre 2024 - 14h30mn45s - Avertissement


// Consignes :
//  1. Ne pas modifier les lignes de code déjà présentes, ajouter vos lignes de code nécessaires
//  2. Utiliser uniquement des instructions RPG natives (pas de SQL, pas d'appels d'API ...)
//  3. Ne prendre que les types "Critique", "Avertissement" et "Suspect"

// Critères d'évaluation :
//  1. Le résultat doit être celui demandé
//  2. Le code présent ne doit pas être modifié
//  3. Le code doit être le plus simple, le plus lisible possible et le plus optimisé.
//  4. Le programme doit s'appeler E2 dans votre bibliothèque TEAMXX

// -----------------------------------------------------------------------------
// Votre profil IBM i : TEAMXX
// -----------------------------------------------------------------------------

ctl-opt actgrp(*new) alwnull(*no);

dcl-f log rename(log:flog);

// Lecture et stockage
Dcl-Ds log_cur likerec(flog);
Dcl-Ds logs    likerec(flog) dim(*AUTO:100);

// Pour formatage date et heure
dcl-pr formatage varchar(20) overload(format_date : format_time);
dcl-s  msg varchar(52) inz ;

// Lecture du fichier des logs
read log log_cur;
dow not %eof;
   // Exclusion des logs <> de la liste
   if log_cur.logtype in %list('Critique' : 'Avertissement' : 'Suspect');
      // Stockage
      logs(*next) = log_cur ;
   endif;
   read log log_cur;
Enddo;

// Tri
sorta(d) logs %fields(logdate : logtime ) ;
// Affichage
for-each log_cur in logs;
   msg = formatage(log_cur.logdate) + ' - ' +
         formatage(log_cur.logtime) + ' - ' +
         log_cur.logtype ;
   dsply msg;
endfor ;

return ;

// Formatage de la date au format 11 Novembre 2024
dcl-proc format_date;
  dcl-s l_datechar varchar(10);
  dcl-s l_mois varchar(15) dim(12) inz static ;
  dcl-s l_inz ind inz(*off) static ;

  dcl-pi *n varchar(20);
     p_date date const;
  end-pi;

  // chargement des libellés, au premier appel
  if not l_inz ;
    l_mois = %list('Janvier' : 'Fevrier' : 'Mars'      : 'Avril'   : 'Mai'      : 'Juin' :
                   'Juillet' : 'Aout'    : 'Septembre' : 'Octobre' : 'Novembre' : 'Decembre');
    l_inz = *on ;
  endif ;

  l_datechar = %char(p_date:*iso);
  // Retourner la date sous le format demandé
  return %subst(l_datechar:9:2) + ' ' + l_mois(%subdt(p_date:*months)) + ' ' +
         %subst(l_datechar:1:4);

End-proc;

// Formatage de l'heure au format 14h30mn45s
dcl-proc format_time;

  dcl-pi *n varchar(20);
     p_time time const ;
  end-pi;
  return %editw(%dec(p_time) : '0  h  mn  s') ;

End-proc; 